import { _decorator, Animation, BoxCollider2D, CCFloat, Collider, Component, director, EPhysics2DDrawFlags, ICollisionEvent, Node, PhysicsSystem, PhysicsSystem2D, RigidBody, RigidBody2D, SkeletalAnimation, v2, v3, Vec2, Vec3 } from 'cc';
import { StateDefine } from './StateDefine';
import { TowardsDirection } from './TowardsDirection';
import { MathUtil } from '../Util/MathUtil';
const { ccclass, property } = _decorator;
let tempVelocity = v2();
let tempDirection: string = null;
@ccclass('Actor')
export class Actor extends Component {
    currentState: StateDefine = null;
    currentTowards: TowardsDirection = null;
    private rigidBody: RigidBody2D = null;
    private collider: BoxCollider2D = null;
    private animation: Animation = null;
    @property(CCFloat)
    linearSpeed: number = 1.0;
    input: Vec2 = v2();
    // actorProperty: ActorProperty = new ActorProperty();
    onLoad() {
        this.rigidBody = this.node.getComponent(RigidBody2D);
        this.collider = this.node.getComponent(BoxCollider2D);
        this.animation = this.node.getChildByName('role').getComponent(Animation);
    }
    start() {
        this.currentState = StateDefine.Idle;
        this.currentTowards = TowardsDirection.Down;
        this.animation.play('Idle-Down');


    }

    update(deltaTime: number) {
        switch (this.currentState) {
            case StateDefine.Idle:
                break;
            case StateDefine.Run:
                this.doMove();
                break;
            case StateDefine.Hit:
                break;
            case StateDefine.Die:
                break;
            default:
                break;
        }
    }
    doMove() {
        const speed = this.input.length() * this.linearSpeed;
        tempVelocity.x = this.input.x * speed;
        tempVelocity.y = this.input.y * speed
        this.rigidBody.linearVelocity = tempVelocity;
    }
    stopMove() {
        this.rigidBody.linearVelocity = Vec2.ZERO;
        
    }
    changeState(deststate: StateDefine) {
        if (this.currentState == StateDefine.Die) {
            return;
        }
        //这个if逻辑没搞懂
        // if (this.currentState == StateDefine.Hit) {
        //     if (deststate != StateDefine.Die && deststate != StateDefine.Hit) {
        //         return;
        //     }
        // }
        if (deststate != StateDefine.Run) {
            //只有切换的一下才速度归零，在update里面一直执行速度归零没有必要
            this.stopMove();
        }
        //更新当前状态，这样update就能根据状态做相应的处理
        this.currentState = deststate;
        //状态改变时，播放相应的动画
        this.changeAnimation();
    }
    //获取方向
    getTowards() {
        return this.currentTowards;
    }
    //切换方向
    setTowards(towards: TowardsDirection) {
        this.currentTowards = towards;
    }
    //切换动画
    changeAnimation(){
        if(this.currentState == StateDefine.Run){
            this.playAnimation('Run', this.currentTowards);
        }
        if(this.currentState == StateDefine.Idle){
            this.playAnimation('Idle', this.currentTowards);
        }
    }
    //播放动画
    playAnimation(state, direction) {
        //设置是否水平翻转
        const scale: Vec3 = (direction === TowardsDirection.Left) ? v3(-1, 1, 1) : v3(1, 1, 1);
        //设置动画名，如果方向为Left就使用Right的动画，然后水平翻转
        const animationName = `${state}-${direction === TowardsDirection.Left ? 'Right' : direction}`;
        this.animation.node.setScale(scale);
        this.animation.play(animationName);
    }
    respawn() {
        this.currentState = StateDefine.Idle;
    }
}


