import { _decorator, BoxCollider2D, CircleCollider2D, Collider2D, Component, director, ERigidBody2DType, instantiate, Node, PolygonCollider2D, Pool, Prefab, resources, RigidBody2D, Size, size, TiledLayer, TiledMap, TiledObjectGroup, v2, v3, Vec2 } from 'cc';
import { ColliderGroup } from './ColliderGroup';
import { GridColliderMap, TileGridBidirectional } from './TileGrid';
const { ccclass, property } = _decorator;

@ccclass('MapController')
export class MapController extends Component {
    private map: TiledMap = null;
    private layer: TiledLayer = null;
    private objectLayer: TiledObjectGroup = null;
    private Obstacle: Node = null;
    private tileSize: Size = null;
    private mapSize: Size = null;
    colliderPool: Pool<Node> = null;
    prefabMap: Map<string, Prefab> = new Map();
    protected onLoad(): void {
        this.map = this.node.getComponent(TiledMap);
        //获取地图的块大小
        this.mapSize = this.map.getMapSize();
        this.Obstacle = this.node.getChildByName('Obstacle');
        this.layer = this.map.getLayer('ground');
        this.objectLayer = this.map.getObjectGroup('test');
        //获取一个块的像素大小
        this.tileSize = this.map.getTileSize();
        //加载碰撞体预制体
        resources.loadDir("Obstacle", Prefab, (err: Error, prefabs: Prefab[]) => {
            if (err) {
                throw err;
            }
            prefabs.forEach(prefab => {
                this.prefabMap[prefab.name] = prefab;
            });
            this.addObstacle(this.layer);
            this.combineCollider();
        })
    }
    protected start(): void {
        this.addCollider(this.objectLayer);
    }
    protected onDestroy(): void {
        this.prefabMap.clear();
    }
    //添加地形碰撞
    addObstacle(layer: TiledLayer) {
        if(!layer){
            return;
        }
        //获取图层的块数（如没有修改，一般就是地图的块数）
        const layerSize = layer.getLayerSize();
        for (let i = 0; i < layerSize.height; i++) {
            for (let j = 0; j < layerSize.width; j++) {
                //获取当前块的gid 
                const gid = layer.getTileGIDAt(j, i);
                if (gid != 0) {
                    const collider = this.getCollider(gid);
                    if (collider) {
                        const prefab = this.prefabMap[collider];
                        let colliderNode = instantiate(prefab);
                        colliderNode.setParent(this.Obstacle);
                        let colliderNodePosition = v3((j-this.mapSize.width/2+0.5) * this.tileSize.width, (this.mapSize.height/2-i-1+0.5)*this.tileSize.height, 0);
                        colliderNode.setPosition(colliderNodePosition);
                    }
                    // else {
                    // }
                }
            }
        }
        //要重新激活节点，位置才会生效
        this.Obstacle.active=false;
        this.Obstacle.active=true;
    }
    combineCollider(){

    }
    //添加对象碰撞
    addCollider(objectLayer: TiledObjectGroup) {
        if(!objectLayer){
            return;
        }
        const objects = objectLayer.getObjects();
        objects.forEach(obj => {
            const node = new Node();
            node.setParent(this.node.getChildByName('test2'));
            node.setPosition(v3(obj.x - 184, obj.y - 320, 0));
            // 根据对象类型创建不同的碰撞体
            switch (obj.type) {
                case 3:
                case 4:
                case 0:
                    node.addComponent(BoxCollider2D);
                    const boxCollider = node.getComponent(BoxCollider2D);
                    boxCollider.size = size(obj.width, obj.height);
                    boxCollider.offset = v2(obj.width / 2, obj.height / 2);
                    break;
                case 5:
                    node.addComponent(BoxCollider2D);
                    const boxCollider0 = node.getComponent(BoxCollider2D);
                    boxCollider0.size = size(obj.width, obj.height);
                    boxCollider0.offset = v2(obj.width / 2, -obj.height / 2);
                    break;
                case 1:
                    node.addComponent(CircleCollider2D);
                    const circleCollider = node.getComponent(CircleCollider2D);
                    circleCollider.radius = Math.max(obj.width / 2, obj.height / 2);
                    // circleCollider.offset = v2(obj.x, obj.y);
                    break;
                case 2:
                    node.addComponent(PolygonCollider2D);
                    const polylineCollider = node.getComponent(PolygonCollider2D);
                    const polylinePoints = obj.points.map(point => v2(point.x, point.y));
                    polylineCollider.points = polylinePoints;
                    // polylineCollider.offset = v2(obj.x-180, obj.y-360);
                    break;
                default:
                    break;
            }
            const collider = node.getComponent(Collider2D);
            collider.group = ColliderGroup.Obstacle;
            collider.apply();
        })
    }
    getCollider(value) {
        const name = TileGridBidirectional.valueToName[value]
        return GridColliderMap[name];
    }
    
}


