import { _decorator, CCFloat, Component, EventTouch, Input, input, math, Node, UITransform, v3, Vec3 } from 'cc';
import { VirtualInput } from '../Input/VirtualInput';
const { ccclass, property } = _decorator;

@ccclass('UIJoystick')
export class UIJoystick extends Component {
    private stickBg: Node = null;
    private thumbnail: Node = null;
    private radius: number = 0;
    initPositon: Vec3 = v3();
    protected onLoad(): void {
        //加载节点触摸事件，只有在节点ContentSize范围内的触摸才会生效
        this.node.on(Input.EventType.TOUCH_START, this.onTouchStart, this);
        //要注意移动事件是起点在里面就会一直生效
        this.node.on(Input.EventType.TOUCH_MOVE, this.onTouchMove, this);
        //重点在范围内会触发END
        this.node.on(Input.EventType.TOUCH_END, this.onTouchEnd, this);
        //终点在范围外会触发CANCEL
        this.node.on(Input.EventType.TOUCH_CANCEL, this.onTouchCancel, this);
        this.stickBg = this.node.getChildByName('StickBg');
        this.thumbnail = this.stickBg.getChildByName('Thumbnail');
        this.radius = this.stickBg.getComponent(UITransform).contentSize.width/2;
    }
    protected onDestroy(): void {
        this.node.off(Input.EventType.TOUCH_START, this.onTouchStart, this);
        this.node.off(Input.EventType.TOUCH_MOVE, this.onTouchMove, this);
        this.node.off(Input.EventType.TOUCH_END, this.onTouchEnd, this);
        this.node.off(Input.EventType.TOUCH_CANCEL, this.onTouchCancel, this);
    }
    start() {
        //初始化位置
        this.initPositon = this.stickBg.worldPosition.clone();
        this.stickBg.active = false;
    }

    //将stickBg的位置设置为当前触摸点的位置
    onTouchStart(eventTouch: EventTouch) {
        this.stickBg.active = true;
        const touchPos = eventTouch.touch.getUILocation();
        this.stickBg.setWorldPosition(touchPos.x, touchPos.y,0);
    }
    //移动Thumbnail的位置
    onTouchMove(eventTouch: EventTouch) {
         // 获取摇杆在 UI 的位置
         let x = eventTouch.touch.getUILocationX();
         let y = eventTouch.touch.getUILocationY();
 
         let worldPosition = new Vec3(x, y, 0);
         let localPosition = v3();
 
         // 将Tumbnail的世界坐标转换为本地坐标
         this.stickBg.inverseTransformPoint(localPosition, worldPosition);
         let thumbnailPosition = v3();
         //计算向量的长度
         let len = localPosition.length();
         //归一化向量（使向量长度变为1）方向
         localPosition.normalize();
         //scaleAndAdd为向量的缩放加法，v3()是被加向量，localPosition是要加上的缩放向量，math.clamp(len, 0, this.radius)为缩放倍数
         //math.clamp(len, 0, this.radius)指的是len的取值范围为0到this.radius，如果len小于0，则取0，如果len大于this.radius，则取this.radius
         Vec3.scaleAndAdd(thumbnailPosition, v3(), localPosition, math.clamp(len, 0, this.radius));

         //设置Thumbnail的位置，到这里UI上的Thumbnail位置已经完成了
         this.thumbnail.setPosition(thumbnailPosition);

         // 将计算的结果赋予给 Input
         VirtualInput.horizontal = this.thumbnail.position.x / this.radius;
         VirtualInput.vertical = this.thumbnail.position.y / this.radius;
         
    }
    //将摇杆归位
    onTouchEnd(eventTouch: EventTouch) {
        this.stickBg.active = false;
        this.stickBg.setWorldPosition(this.initPositon);
        this.thumbnail.setPosition(Vec3.ZERO);
        VirtualInput.vertical=0;
        VirtualInput.horizontal=0;
    }
    onTouchCancel(eventTouch: EventTouch) {
        this.stickBg.active = false;
        this.stickBg.setWorldPosition(this.initPositon);
        this.thumbnail.setPosition(Vec3.ZERO);
        VirtualInput.vertical=0;
        VirtualInput.horizontal=0;

    }
}


